@@ -937,7 +937,7 @@ GtkWidget *gpdf_window_new (void)
 
 	gpdf_window = GPDF_WINDOW (g_object_new (GPDF_TYPE_WINDOW,
 						 "win_name", "gpdf",
-						 "title", _("GNOME PDF Viewer"),
+						 "title", _("PDF Viewer"),
 						 "default-width", 600,
 						 "default-height", 600,
 						 NULL));
