@@ -29,7 +29,7 @@
 #include "GfxState.h"
 #include "GfxFont.h"
 #include "GPOutputDev.h"
-#include "FontFile.h"
+#include "FoFiTrueType.h"
 
 #ifdef HAVE_FONT_EMBEDDING
 #  include "gpdf-g-switch.h"
@@ -67,6 +67,10 @@ const struct {
   {"ZapfDingbats",          "Dingbats Regular"}      //"ITC Zapf Dingbats"}
 };
 
+static void fileWrite(void *stream, char *data, int len) {
+  fwrite(data, 1, len, (FILE *)stream);
+}
+
 GPOFontMap::GPOFontMap() : embeddedFonts(NULL)
 {
   initBase14Fonts();
@@ -247,8 +251,8 @@ GnomeFontFace *GPOFontMap::getFontFaceEmbedded(GfxFont *font)
       fd = g_file_open_tmp ("gpdf-ttf-XXXXXX", &temp_name, NULL);
       FILE *f = fdopen (fd, "wb");
 
-      TrueTypeFontFile *ff = new TrueTypeFontFile((char *)contents, length);
-      ff->writeTTF(f);
+      FoFiTrueType *ff = FoFiTrueType::make((char *)contents, length);
+      ff->writeTTF(&fileWrite, f);
       delete ff;
       g_free(contents);
 
@@ -668,7 +672,13 @@ void GPOutputDev::drawString(GfxState *state, GString *s) /* FIXME */
     return;
 
   gnome_print_gsave(gpc);
-  gnome_print_concat(gpc, state->getTextMat());
+
+  gdouble fontmat[6];
+  memcpy(fontmat, state->getTextMat(), 6 * sizeof(gdouble));
+  fontmat[0] *= state->getHorizScaling();
+  fontmat[2] *= state->getHorizScaling();
+
+  gnome_print_concat(gpc, fontmat);
   gnome_print_moveto(gpc, text_pos_x, text_pos_y);
 
   GnomeGlyphList *ggl = gnome_glyphlist_new();
@@ -696,7 +706,6 @@ void GPOutputDev::drawString(GfxState *state, GString *s) /* FIXME */
     dx = dx * state->getFontSize() + state->getCharSpace();
     if (n == 1 && *p == ' ')
       dx += state->getWordSpace();
-    dx *= state->getHorizScaling();
     dy *= state->getFontSize ();
 
     DBG (if (uLen > 1)
@@ -732,18 +741,25 @@ void GPOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   GfxRGB maskColor;
   state->getFillRGB(&maskColor);
 
+  guchar red = (guchar)(maskColor.r * 255.999);
+  guchar green = (guchar)(maskColor.g * 255.999);
+  guchar blue = (guchar)(maskColor.b * 255.999);
+
   guchar *pixbuf = (guchar *)g_malloc(width * height * 4);
   g_return_if_fail(pixbuf != NULL);
 
   guchar *p = pixbuf;
+  guchar alpha;
   for (gint i = 0; i < width * height; ++i) {
-    guchar alpha;
     imgStr->getPixel(&alpha);
 
-    *p++ = (guchar)(maskColor.r * 255.999);
-    *p++ = (guchar)(maskColor.g * 255.999);
-    *p++ = (guchar)(maskColor.b * 255.999);
-    *p++ = alpha ? 0 : 255;
+    *p++ = red;
+    *p++ = green;
+    *p++ = blue;
+    if (invert)
+      *p++ = alpha ? 255 : 0;
+    else
+      *p++ = alpha ? 0 : 255;
   }
 
   gnome_print_rgbaimage(gpc, pixbuf, width, height, width * 4);
@@ -768,10 +784,9 @@ void GPOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   g_return_if_fail(pixbuf != NULL);
 
   guchar *p = pixbuf;
+  Guchar pix[gfxColorMaxComps];
+  GfxRGB rgb;
   for (gint i = 0; i < width * height; ++i) {
-    Guchar pix[gfxColorMaxComps];
-    GfxRGB rgb;
-
     imgStr->getPixel(pix);
     colorMap->getRGB(pix, &rgb);
 
